/*
 * logging.cpp
 *
 * Functions for logging
 *
 * Ulrik Petersen
 * Created: 11/21-2002
 * Last update: 11/21-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <emdf.h>
#include <logging.h>

#if EMDROS_LOGGING
#include <time.h>
#include <fstream>
#include <cstdlib>
#include <cstring>
#include <emdros-lconfig.h>
#endif


#if !(EMDROS_LOGGING)
void log_write(std::string function_name, std::string message)
{
  // Do nothing
}

void log_write_time(std::string function_name, std::string message)
{
  // Do nothing
}
#else // So we must do logging


#define BUFFER_LENGTH  (30)
void get_time(std::string& mytime)
{
  // Allocate time structure
  time_t* pTime = (time_t*) malloc(sizeof(time_t));

  // Allocate string buffer
  char* szTime = (char*) malloc(BUFFER_LENGTH);

  // Get localt time
  time(pTime);

  // Convert to string
#if LINUX
  ctime_r(pTime, szTime);
#else // This should be SUN
  ctime_r(pTime, szTime, BUFFER_LENGTH);
#endif

  // Chop newline
  szTime[strlen(szTime)-1] = '\0';

  // Set mytime
  mytime = szTime;

  // Free memory
  free(szTime);
  free(pTime);
}

#define OPEN_LOG(FSTR) std::ofstream FSTR(EMDROS_LOGFILE, std::ios_base::app); \
   if (!FSTR) return; 

void log_write(std::string function_name, std::string message)
{
  OPEN_LOG(fout);
  fout << function_name << ":" << message << std::endl;
}

void log_write_time(std::string function_name, std::string message)
{
  OPEN_LOG(fout);
  std::string mytime;
  get_time(mytime);
  fout << function_name << " " << mytime << ":" << message << std::endl;
}


#endif


