/*
 * pgemdfdb.cpp
 *
 * PostgreSQL EMdF database base class
 *
 * Ulrik Petersen
 * Created: 1/27-2001
 * Last update: 6/21-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <pgemdfdb.h>
#include <pgconn.h>
#include <encryption.h>
#include <utils.h>

PgEMdFDB::PgEMdFDB(std::string host, std::string user, std::string passwd, std::string database_name) : m_host(host), m_user(user), m_passwd(passwd)
{
  pConn = new PgEMdFConnection(host, user, passwd, database_name);
  encrypt_string(m_passwd);
}


PgEMdFDB::~PgEMdFDB()
{
}

bool PgEMdFDB::UseDatabase(const std::string& db_name)
{
  if (pConn == 0)
    return false;
  else {
    std::string NewDbName;
    ConvertDBName(db_name, NewDbName);

    std::string unencrypted_password = m_passwd;
    unencrypt_string(unencrypted_password);

    delete pConn;
    pConn = new PgEMdFConnection(m_host, m_user, unencrypted_password, NewDbName);

    zero_fill_string(unencrypted_password);

    // Call base class
    EMdFDB::UseDatabase(db_name);

    return pConn->ConnectionOk();
  }
}

