/*
 * mql_R.cpp
 *
 * MQL retrieval functions
 *
 * Ulrik Petersen
 * Created: 3/16-2001
 * Last update: 6/24-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#include <mql_symboltable.h>
#include <mql_R.h>
#include <mql_extern.h>
#include <mql_sheaf.h>
#include <mql_utility.h>
#include <mql_mo_nr.h>
#include <inst.h>
#include <assert.h>
#include <vector>
#include <string>
#include <assert.h>
#include <algorithm>
#include <logging.h>



//
// Forward declarations
//
CInst* R_inst_block_string(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlock_string* pBlock_string);
CInst* R_inst_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlocks* pBlocks);
CSheaf* R_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst* pInst, CBlocks* pBlocks);
CList_of_straws* R_block_string(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CBlock_string* pBlock_string);
CList_of_straws* R_block_str(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CBlock_str* pBlock_str);
CMo_nr* R_object_block_first(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CObject_block_first* pObject_block_first);
void R_object_reference_declaration_DB(CMQL_execution_environment *pEE, id_d_t object_id_d, short int index);
void R_object_reference_declaration_inst(CMQL_execution_environment *pEE, id_d_t object_id_d, CMQLObject *pMQLObject, short int index);
CInst* R_inst_opt_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlocks* pOpt_blocks);
bool R_focus_retrieval(CMQL_execution_environment *pEE, eRetrieval retrieval);
bool R_retrieval_retrieval(CMQL_execution_environment *pEE, eRetrieval retrieval);
bool R_features(CMQL_execution_environment *pEE, CMQLObject* pObj, CFFeatures* pFFeatures);
CSheaf* R_opt_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst* pInst, CBlocks* pOpt_blocks);
CList_of_straws*  R_rest_of_block_str(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CRest_of_block_str* pRest_of_block_str);
void R_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CBlock* pBlock, CList_of_mo_nr* pLOMN);
void R_opt_gap_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, COpt_gap_block* pOpt_gap_block, CList_of_mo_nr* pLOMN);
bool R_retrieval_gap_retrieval(CMQL_execution_environment *pEE, eRetrieval gap_retrieval);
bool R_focus_gap_retrieval(CMQL_execution_environment *pEE, eRetrieval gap_retrieval);
void R_gap_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CGap_block* pGap_block, CList_of_mo_nr* pLOMN);
void R_object_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CObject_block* pObject_block, CList_of_mo_nr* pLOMN);


CSheaf* R_topograph(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CTopograph* pTopograph)
{
  // Make inst
  CInst* pInst;
  pInst = R_inst_blocks(pEE, U, pTopograph->get_blocks());
  
  // Get sheaf
  CSheaf* pSheaf;
  pSheaf = R_blocks(pEE, U, Su, pInst, pTopograph->get_blocks());

  // Delete inst
  if (!pInst->is_aggregate())
    delete pInst;
  
  // Return result
  return pSheaf;
}

//
// Local helper function
//
CInst* R_inst(CMQL_execution_environment *pEE, const CSet_of_monad_ms& Su, CObject_block_common *object_block)
{
  // Declare variable
  CInst* pInst;

  // Choose whether to use object's own inst or to calculate the inst
  if ((pInst = object_block->m_inst) != 0) {
    //
    // Use object's own inst.
    // This has been made before letting R_topograph loose on the query.
    //
  } else {
    //
    // Make inst from scratch
    //

    // Write to log
    // LOG_WRITE_TIME("mql_R.cpp:R_inst()", "Starting inst-creation.");

    // Get list of feature names to get
    const std::list<feature_info>& feature_infos = object_block->get_object()->get_feature_infos();

    // Get pre-query string
    std::string pre_query_string("");
    pre_query_string = object_block->get_pre_query_string();

    // Make inst
    pInst = new CInst(object_block->get_object()->get_feature_names());

    // Calculate set of monads to pass to GetInst
    const CSet_of_monad_ms& characteristic_set = Su;

    // Get inst
    if (!pEE->pDB->GetInst(object_block->get_object_type_name(), 
			   characteristic_set, 
			   pEE->m_all_m_1,
			   pre_query_string,
			   feature_infos,
			   *pInst)) {
      throw CEMdFDBDBError();
    }

    // Write to log
    // LOG_WRITE_TIME("mql_R.cpp:R_inst()", "Inst-creation finished.");
  } 
  return pInst;
}

CInst* R_inst_block_string(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlock_string* pBlock_string)
{
  CObject_block_first *object_block_first;
  object_block_first = pBlock_string->get_leftmost_object_block_first();
  return R_inst(pEE, U, object_block_first);
}

CInst* R_inst_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlocks* pBlocks)
{
  CObject_block_first *object_block_first;
  object_block_first = pBlocks->get_leftmost_object_block_first();
  return R_inst(pEE, U, object_block_first);
}

CSheaf* R_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst* pInst, CBlocks* pBlocks)
{
  // Declare and initialize i
  pInst->set_current_universe(U);
  CInst::const_iterator i(pInst->begin());

  // Declare and initialize pResult
  CList_of_straws* pResult;
  pResult = new CList_of_straws;
 
  // Declare and initialize bSelf_success
  bool bSelf_success;
  bSelf_success = false;

  // Treat the block_string
  CList_of_straws* pLOS;
  bool bLOSSuccess;
  CInst::const_iterator cend(pInst->end());
  do {
    pLOS = R_block_string(pEE, U, Su, i, pInst, 0, pBlocks->get_block_string());
    bLOSSuccess = pLOS->m_success;
    if (bLOSSuccess) {
      bSelf_success = true;
      pResult->append_and_subsume(pLOS);
    } else {
      delete pLOS;
    }
    if (i != cend)
      pInst->next(i);
  } while (bLOSSuccess && i != cend);

  // Construct sheaf
  CSheaf* pSheaf;
  if (bSelf_success) {
    // We were a success
    pSheaf = new CSheaf(pResult);
  } else {
    // We failed.  Delete pResult now.
    delete pResult;
    pSheaf = new CSheaf(true);
  }
  return pSheaf;
}

CList_of_straws* R_block_string(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CBlock_string* pBlock_string)
{
  // Decide syntactic incarnation
  if (pBlock_string->is_block_str()) {
    return R_block_str(pEE, U, Su, i, pInst, limit, pBlock_string->get_block_str());
  } else {
    CInst::const_iterator instend = pInst->end();
    while (true) {
      CList_of_straws* pLOS1;
      pLOS1 = R_block_str(pEE, U, Su, i, pInst, limit, pBlock_string->get_block_str());
      CList_of_straws* pResult = new CList_of_straws();
      if (!pLOS1->m_success) {
	delete pLOS1;
	pResult->set_success(false);
	return pResult;
      } else {
	bool bSelf_success = false;
	if (pLOS1->is_empty()) {
	  CInst* pInstping = R_inst_block_string(pEE, U, pBlock_string->get_block_string());
	  pInstping->set_current_universe(U);
	  CInst::const_iterator iping(pInstping->begin());
	  CInst::const_iterator ipingend(pInstping->end());
	  long limitping = pBlock_string->get_power()->get_limit();
	  CList_of_straws* pLOS2;
	  bool bLOS2Success;
	  do {
	    pLOS2 = R_block_string(pEE, U, Su, iping, pInstping, limitping, pBlock_string->get_block_string());
	    bLOS2Success = pLOS2->m_success;
	    if (bLOS2Success) {
	      bSelf_success = true;
	      pResult->append_and_subsume(pLOS2);
	    } else {
	      delete pLOS2;
	    }
	    if (iping != ipingend)
	      pInstping->next(iping);
	  } while (bLOS2Success && iping != ipingend);
	  if (!pInstping->is_aggregate())
	    delete pInstping;
	} else {
	  SheafIterator si = pLOS1->iterator();
	  while (si.hasNext()) {
	    monad_m strawlast = (si.current())->get_last();
	    CSet_of_monad_ms Uping; 
	    Restrict(U, strawlast+1, Uping);
	    CSet_of_monad_ms Suping;
	    Restrict(Su, strawlast+1, Suping);
	    if (!Uping.is_empty() && !Suping.is_empty()) {
	      CInst* pInstping = R_inst_block_string(pEE, Uping, pBlock_string->get_block_string());
	      pInstping->set_current_universe(Uping);
	      CInst::const_iterator iping(pInstping->begin());
	      CInst::const_iterator ipingend(pInstping->end());
	      long limitping = pBlock_string->get_power()->get_limit();
	      CList_of_straws* pLOS3;
	      bool bLOS3Success;
	      do {
		pLOS3 = R_block_string(pEE, Uping, Suping, iping, pInstping, limitping, pBlock_string->get_block_string());
		bLOS3Success = pLOS3->m_success;
		if (bLOS3Success) {
		  bSelf_success = true;
		  pLOS3->Join(si.current());
		  pResult->append_and_subsume(pLOS3);
		} else {
		  delete pLOS3;
		}
		if (iping != ipingend)
		  pInstping->next(iping);
	      } while (bLOS3Success && iping != ipingend);
	      if (!pInstping->is_aggregate())
		delete pInstping;
	    }
	    si.next();
	  }
	}
	delete pLOS1;
	if (bSelf_success) {
	  pResult->set_success(true);
	  return pResult;
	}
      }
      delete pResult;
      if (i != instend)
	pInst->next(i);
    }
  }
}

CList_of_straws* R_block_str(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CBlock_str* pBlock_str)
{
  bool bSuccess = false;
  bool bReturn = false;
  CInst::const_iterator instend(pInst->end());
  while (true) {
    // Which syntactic incarnation?
    if (pBlock_str->is_object_block_first()) {
      //
      // syntactic incarnation is object_block_first
      //

      // Get result of object_block_first
      CMo_nr* pMo_nr = R_object_block_first(pEE, U, Su, i, pInst, limit, pBlock_str->get_object_block_first());

      // Declare variables
      CList_of_straws* pLOS = 0;

      if (pMo_nr->is_NIL_mo()) {
	// If it was a NIL_mo, the object_block_first failed.
	// Return ( < > , bSuccess )
	pLOS = new CList_of_straws();
	bReturn = true;
      } else if (!pMo_nr->get_nr()) {
	// If it was not NIL_mo and we are to retrieve it,
	// Then Return ( < mo > , true )
	pLOS = new CList_of_straws();
	bSuccess = true;
	bReturn = true;
	CStraw* pStraw = new CStraw();
	pStraw->append(pMo_nr->get_copy_of_mo());
	pLOS->append(pStraw);
      } else {
	// Now we know that we are a success,
	// but we should not return yet.
	bReturn = false;
	bSuccess = true;
      }

      // Clean up mo_nr
      delete pMo_nr;

      // If we should return, set pLOS's success to bSuccess and return.
      if (bReturn) {
	pLOS->set_success(bSuccess);
	return pLOS;
      }
    } else {
      //
      // syntactic incarnation was 
      // object_block_first rest_of_block_str
      //

      // Get result of object_block_first
      CMo_nr* pMo_nr = R_object_block_first(pEE, U, Su, i, pInst, limit, pBlock_str->get_object_block_first());

      // Declare variables
      CList_of_straws* pLOS = 0;

      // Initialize bSuccess and bReturn
      bSuccess = false;
      bReturn = false;

      // If it was NIL_mo, it means that R_object_block_first
      // has exhausted all possibilities
      if (pMo_nr->is_NIL_mo()) {
	// Return ( < > , false )
	pLOS = new CList_of_straws();
	bSuccess = false;
	bReturn = true;
      } else {
	// Calculate Sm
	monad_m Sm = pMo_nr->get_const_reference().get_last() + 1;

	// Get result of rest_of_block_str
	CList_of_straws* pLOSrest;
	pLOSrest = R_rest_of_block_str(pEE, U, Su, Sm, pBlock_str->get_rest_of_block_str());
	// If we were a success, then	
	if (pLOSrest->m_success) {
	  // If we are not to retrieve the object_block_first
	  if (pMo_nr->get_nr()) {
	    // Simply return the LOS from pLOSrest
	    pLOS = pLOSrest;
	    bSuccess = true;
	    bReturn = true;
	  } else {
	    // Return Join( < mo > , LOS from pLOSrest )
	    CStraw* pStraw = new CStraw();
	    pStraw->append(pMo_nr->get_copy_of_mo());
	    pLOS = pLOSrest;
	    pLOS->Join(pStraw);
	    delete pStraw;
	    bSuccess = true;
	    bReturn = true;
	  }
	} else { // rest_of_block_str was not a success
	  // Clean up
	  delete pLOSrest;
	}
      }

      // Clean up mo_nr
      delete pMo_nr;

      // If we should return, set success and return.
      if (bReturn) {
	pLOS->set_success(bSuccess);
	return pLOS;
      }
    }

    // Advance i iterator
    if (i != instend)
      pInst->next(i);
  }
}

CMo_nr* R_object_block_first(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst::const_iterator& i, CInst* pInst, long limit, CObject_block_first* pObject_block_first)
{
  // Calculate first or last
  eFirstLast fl = pObject_block_first->get_first_last();

  // Get inst's end
  CInst::const_iterator instend(pInst->end());

  while (true) {
    // If we have gone too far, we must fail
    if (i == instend) {
      CMatched_object* pNIL_mo = new CMatched_object(kMOKNIL_mo);
      CMo_nr* pMo_nr = new CMo_nr(pNIL_mo, true);
      return pMo_nr;
    }

    // Get the object of type "type" which is the "i'th" 
    // within the universe
    CInstObject *pInstObj = i->second;
    id_d_t id_d_O = pInstObj->get_id_d();

    // Get set of monads for object
    const CSet_of_monad_ms& som = pInstObj->get_monads();

    // Update local symbol table with information
    CMQLObject *pObj = pObject_block_first->get_object();
    pObj->retrieve_feature_values(pInstObj);

    // Get local first variables, so we don't need to
    // call first() every time.
    monad_m som_first = som.first();
    monad_m su_first = Su.first();

    //
    // Check that rules are obeyed
    //
    bool bDoObject = true;

    // Check that limit is obeyed
    if (limit != 0) {
      bDoObject = (som_first <= (su_first + limit - 1));
    } else if (limit == 0) {
      bDoObject = true;
    }

    // Get local last variables, so we don't need to
    // call last() every time.
    monad_m som_last = som.last();
    monad_m su_last = Su.last();

    // Check that first/last are obeyed
    if (bDoObject) {
      switch (fl) {
      case kNoFirstLast:
	bDoObject = true;
	break;
      case kFirst:
	bDoObject = som_first == su_first;
	break;
      case kLast:
	bDoObject = som_last == su_last;
	break;
      case kFirstAndLast:
	bDoObject = som_first == su_first && som_last == su_last;
	break;
      default:
	assert(false);
	break;
      }
    }

    // Check that object is part_of Su
    if (bDoObject) {
      bDoObject = som.part_of(Su);
    }

    // Check that feature-constraints are met
    if (bDoObject) {
      bDoObject = R_features(pEE, pObject_block_first->get_object(), 
			     pObject_block_first->get_feature_constraints());
    }

    // Did we pass the test?
    if (bDoObject) {
      // So we passed the test
      
      // Update symbol table with object reference declaration
      R_object_reference_declaration_inst(pEE, id_d_O, pObject_block_first->get_object(), pObject_block_first->get_object_reference_index());
      // R_object_reference_declaration_DB(pEE, id_d_O, pObject_block_first->get_object_reference_index());
      
      // Make new universe (O.first()..O.last())
      CSet_of_monad_ms Uping(som_first, som_last);
      //Hat(som, Uping);
      
      // Calculate new inst
      CInst* pInstping;
      pInstping = R_inst_opt_blocks(pEE, Uping, pObject_block_first->get_opt_blocks());
      
      // Compute the inner blocks
      CSheaf* pInner;
      try {
	pInner = R_opt_blocks(pEE, Uping, som, pInstping, pObject_block_first->get_opt_blocks());
      } catch (CEMdFDBException e) {
	if (pInstping != 0 && !pInstping->is_aggregate())
	  delete pInstping;
	throw e;
      }
      
      // Clean up pInstping
      if (pInstping != 0 && !pInstping->is_aggregate())
	delete pInstping;
      
      // Only return the object if the inner was empty or did not fail
      if (pInner == 0 || !pInner->is_fail()) {
	CMatched_object* pO;
	eRetrieval retrieval = pObject_block_first->get_retrieval();
	bool bNoRetrieve = R_retrieval_retrieval(pEE, retrieval);
	bool bFocus = R_focus_retrieval(pEE, retrieval);
	pO = new CMatched_object(id_d_O, pObject_block_first->get_object_type_index(), som, bFocus, pInner);
	CMo_nr* pMo_nr = new CMo_nr(pO, bNoRetrieve);
	return pMo_nr;
      } else {
	// Clean up pInner
	delete pInner;
      }
    } 
    
    // Advance progress counter
    pEE->progress();

    // Advance iterator
    if (i != instend)
      pInst->next(i);
  }
}

void R_object_reference_declaration_DB(CMQL_execution_environment *pEE, id_d_t object_id_d, short int index)
{
  // Only do something to non-empty object reference declarations
  if (index != StrSTEPairVec::no_element) {
    // Get symbol table entry
    CMQLSymbolTableEntry* pSTE = pEE->pSymbolTable->get(index);
    assert(pSTE != 0);
  
    // Associate with object id_d
    pSTE->set_id_d(object_id_d);

    // Retrieve features that will be used later
    pSTE->set_features(pEE->pDB);
  }
}

void R_object_reference_declaration_inst(CMQL_execution_environment *pEE, id_d_t object_id_d, CMQLObject *pMQLObject, short int index)
{
  // Only do something to non-empty object reference declarations
  if (index != StrSTEPairVec::no_element) {
    // Get symbol table entry
    CMQLSymbolTableEntry* pSTE = pEE->pSymbolTable->get(index);
    assert(pSTE != 0);
  
    // Associate with object id_d
    pSTE->set_id_d(object_id_d);

    // Retrieve features that will be used later
    pSTE->set_features(pMQLObject);
  }
}

CInst* R_inst_opt_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, CBlocks* pOpt_blocks)
{
  CInst* pResult;
  // Is it a blocks?
  if (pOpt_blocks != 0) {
    // Yes, Return R_inst_blocks on the blocks
    pResult = R_inst_blocks(pEE, U, pOpt_blocks);
  } else {
    // No, return empty inst
    /*
    std::vector<std::string> dummy_feature_names;
    pResult = new CInst(dummy_feature_names);
    */
    pResult = 0;
  }
  return pResult;
}

bool R_focus_retrieval(CMQL_execution_environment *pEE, eRetrieval retrieval)
{
  return retrieval == kRetrieveFocus;
}

bool R_retrieval_retrieval(CMQL_execution_environment *pEE, eRetrieval retrieval)
{
  return retrieval == kNoRetrieve;
}

bool R_feature_comparison(CMQL_execution_environment *pEE, CMQLObject* pObj, CFeature_comparison* pFeature_comparison)
{
  // Declare result
  bool bResult;

  // Get left value
  const CEMdFValue *left_value;
  left_value = pObj->get_feature_value(pFeature_comparison->get_feature_index());

  // Compare
  bResult = pFeature_comparison->compare(pEE, left_value);


  // Return result
  return bResult;
}

bool R_ffactor(CMQL_execution_environment *pEE, CMQLObject* pObj, CFFactor* pFFactor)
{
  if (pFFactor->is_NOT()) {
    return !R_ffactor(pEE, pObj, pFFactor->get_ffactor());
  } else if (pFFactor->is_parenthesis()) {
    return R_features(pEE, pObj, pFFactor->get_ffeatures());
  } else { // is_feature_comparison() == true
    return R_feature_comparison(pEE, pObj, pFFactor->get_feature_comparison());
  }
}

bool R_fterm(CMQL_execution_environment *pEE, CMQLObject* pObj, CFTerm* pFTerm)
{
  if (pFTerm->is_ffactor()) {
    return R_ffactor(pEE, pObj, pFTerm->get_ffactor());
  } else {
    bool bFTerm = R_fterm(pEE, pObj, pFTerm->get_fterm());
    if (!bFTerm)
      return false;
    else {
      return R_ffactor(pEE, pObj, pFTerm->get_ffactor());
    }
  }
}

bool R_features(CMQL_execution_environment *pEE, CMQLObject* pObj, CFFeatures* pFFeatures)
{
  if (pFFeatures == 0)
    return true;
  else {
    if (pFFeatures->is_fterm()) {
      return R_fterm(pEE, pObj, pFFeatures->get_fterm());
    } else {
      bool bFFeatures = R_features(pEE, pObj, pFFeatures->get_ffeatures());
      if (bFFeatures)
	return true;
      else {
	return R_fterm(pEE, pObj, pFFeatures->get_fterm()); // || false
      }
    }
  }
}

CSheaf* R_opt_blocks(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, CInst* pInst, CBlocks* pOpt_blocks)
{
  // Decide syntactic incarnation
  if (pOpt_blocks != 0) {
    // It was block
    return R_blocks(pEE, U, Su, pInst, pOpt_blocks);
  } else {
    // It was /* empty */
    // Return // < >
    return 0;
  }
}

CList_of_straws* R_rest_of_block_str(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CRest_of_block_str* pRest_of_block_str)
{
  // Decide syntactic incarnation
  if (pRest_of_block_str->is_block()) {
    //
    // pRest_of_block_string is "block"
    //

    // Get block's result
    CList_of_mo_nr* pLOMN;
    pLOMN = new CList_of_mo_nr();
    R_block(pEE, U, Su, Sm, pRest_of_block_str->get_block(), pLOMN);

    // Calculate result
    bool bSuccess;
    CList_of_straws* pLOS = new CList_of_straws();
    if (pLOMN->has_NIL_mo()) {
      // Return ( < > , false )
      bSuccess = false;
    } else if (pLOMN->has_EMPTY_mo()) {
      // Return ( < > , true )
      bSuccess = true;
    } else {
      // We are a success
      bSuccess = true;

      // Iterate through lomn
      CList_of_mo_nr::const_iterator ci(pLOMN->begin());
      CList_of_mo_nr::const_iterator cend(pLOMN->end());
      while (ci != cend) {
	// Are we to retrieve it?
	if (!ci->get_nr()) {
	  // pLOS += < < mo > > 
	  CStraw* pStraw;
	  pStraw = new CStraw();
	  pStraw->append(ci->get_copy_of_mo());
	  pLOS->append(pStraw);
	}
	++ci;
      }
    }

    // Clean up pLOMN
    delete pLOMN;

    // Return result
    pLOS->set_success(bSuccess);
    return pLOS;
  } else {
    //
    // pRest_of_block_string is "block1  rest_of_block_str1"
    //

    // Get block's result
    CList_of_mo_nr* pLOMN;
    pLOMN = new CList_of_mo_nr();
    
    R_block(pEE, U, Su, Sm, pRest_of_block_str->get_block(), pLOMN);

    // Make initial empty LOS
    CList_of_straws* pLOS;
    pLOS = new CList_of_straws();

    // Return now if we failed
    if (pLOMN->has_NIL_mo()) {
      // Return ( < > , false )
      pLOS->set_success(false);
      delete pLOMN;
      return pLOS;
    }

    // Iterate through lomn
    bool bSelfSuccess = false;
    CList_of_mo_nr::const_iterator ci(pLOMN->begin());
    CList_of_mo_nr::const_iterator cend(pLOMN->end());
    while (ci != cend) {
      //
      // Calculate Sm'
      monad_m Smping;
      if (ci->is_EMPTY_mo()) {
	// Sm' = Sm
	Smping = Sm;
      } else {
	// Sm' = mo.last() + 1;
	Smping = ci->get_const_reference().get_last() + 1;
      }
      
      // Get the rest.  This is a recursive call to ourselves.
      bool bDeleteLOSrest = true;
      CList_of_straws* pLOSrest = R_rest_of_block_str(pEE, U, Su, Smping, pRest_of_block_str->get_rest_of_block_str());

      // Only add if rest was success
      if (pLOSrest->m_success) {
	// Now we know that we ourselves are a success
	bSelfSuccess = true;

	// Are we not to retrieve the mo?
	if (ci->get_nr()) {
	  // Yes, we are not to retrieve the mo.
	  // Result := Result + rest;
	  pLOS->append_and_subsume(pLOSrest);
	  bDeleteLOSrest = false;
	} else {
	  // No, we are to retrieve the mo
	  // Result := Result + Join( < mo >, rest );
	  CStraw* pStraw;
	  pStraw = new CStraw();
	  pStraw->append(ci->get_copy_of_mo());
	  pLOSrest->Join(pStraw);
	  delete pStraw;
	  pLOS->append_and_subsume(pLOSrest);
	  bDeleteLOSrest = false;
	}
      }

      // Delete the list of straws if necessary
      if (bDeleteLOSrest) {
	delete pLOSrest;
      }

      // Advance iterator
      ++ci;
    }

    // Clean up pLOMN
    delete pLOMN;

    // Return result
    pLOS->set_success(bSelfSuccess);
    return pLOS;
  }
}

void R_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CBlock* pBlock, CList_of_mo_nr* pLOMN)
{
  // Advance progress counter
  pEE->progress();

  // Choose based on kind
  switch (pBlock->get_kind()) {
  case kOptGapBlock:
    R_opt_gap_block(pEE, U, Su, Sm, pBlock->get_opt_gap_block(), pLOMN);
    break;
  case kGapBlock:
    R_gap_block(pEE, U, Su, Sm, pBlock->get_gap_block(), pLOMN);
    break;
  case kObjectBlock:
    R_object_block(pEE, U, Su, Sm, pBlock->get_object_block(), pLOMN);
    break;
  }
}

void R_gap(CMQL_execution_environment *pEE, const CSet_of_monad_ms& Su, monad_m Sm, eRetrieval retrieval, CBlocks* pOpt_blocks, eMOKind mo_kind, CList_of_mo_nr* pLOMN)
{
  monad_m m;
  if (Su.gap_exists(Sm, m)) {
    bool bFocus;
    bool bNoRetrieve;

    // Calculate focus
    bFocus = R_focus_gap_retrieval(pEE, retrieval);

    // Calculate NoRetrieve
    bNoRetrieve = R_retrieval_gap_retrieval(pEE, retrieval);

    // Make somn to be Sm..m
    CSet_of_monad_ms somn(Sm,m);

    //
    // Calculate opt_blocks
    //
    // Calculate new inst
    CInst* pInst;
    pInst = R_inst_opt_blocks(pEE, somn, pOpt_blocks);
      
      // Compute the inner blocks
    CSheaf* pInner = 0;
    try {
      pInner = R_opt_blocks(pEE, somn, somn, pInst, pOpt_blocks);
    } catch (CEMdFDBException e) {
      if (pInst != 0 && !pInst->is_aggregate())
	delete pInst;
      throw e;
    }
      
    // Clean up pInst
    if (pInst != 0 && !pInst->is_aggregate())
      delete pInst;
      
      // If opt_blocks is a fail, we return < kNIL_mo, false >
    if (pInner != 0 && pInner->is_fail()) {
      // Clean up
      delete pInner;

      // Make < kNIL_mo, true >
      CMatched_object* pMo;
      pMo = new CMatched_object(kMOKNIL_mo);
      CMo_nr mo_nr(pMo, true);

      // Append to LOMN
      pLOMN->append(mo_nr);
    } else { // Else we return a real matched_object
      // Make [ somn bFocus pInner ]
      CMatched_object* pMo;
      pMo = new CMatched_object(somn, bFocus, pInner);
      CMo_nr mo_nr(pMo, bNoRetrieve);
      pLOMN->append(mo_nr);
    }
  } else { // If there was no gap, we return < mo_kind , true >
    CMatched_object* pMo;
    pMo = new CMatched_object(mo_kind);
    CMo_nr mo_nr(pMo, true);
    pLOMN->append(mo_nr);
  }
}

void R_opt_gap_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, COpt_gap_block* pOpt_gap_block, CList_of_mo_nr* pLOMN)
{
  eRetrieval retrieval;
  retrieval = pOpt_gap_block->get_retrieval();
  R_gap(pEE, Su, Sm, retrieval, pOpt_gap_block->get_opt_blocks(), kMOKEMPTY_mo, pLOMN);
}

bool R_retrieval_gap_retrieval(CMQL_execution_environment *pEE, eRetrieval gap_retrieval)
{
  if (gap_retrieval == kRetrieve
      || gap_retrieval == kRetrieveFocus)
    return false;
  else
    return true;
}

bool R_focus_gap_retrieval(CMQL_execution_environment *pEE, eRetrieval gap_retrieval)
{
  return gap_retrieval == kRetrieveFocus;
}

void R_gap_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CGap_block* pGap_block, CList_of_mo_nr* pLOMN)
{
  eRetrieval retrieval;
  retrieval = pGap_block->get_retrieval();
  R_gap(pEE, Su, Sm, retrieval, pGap_block->get_opt_blocks(), kMOKNIL_mo, pLOMN);
}

void R_object_block_DB(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CObject_block* pObject_block, CList_of_mo_nr* pLOMN)
{
  // Get all the O's beginning at Sm (Start monad)
  IDDList idd_list;
  if (!pEE->pDB->GetObjectsStartingAtSm(pObject_block->get_object_type_name(), 
				      Sm, 
				      idd_list))
    throw CEMdFDBDBError();

  IDDList::const_iterator ci(idd_list.begin());
  IDDList::const_iterator cend(idd_list.end());
  while (ci != cend) {
    // Get id_d of O
    id_d_t id_d_O = *ci;

    // Get last/first_and_last
    eFirstLast fl = pObject_block->get_first_last();

    // Get set of monads for object
    CSet_of_monad_ms som;
    if (!pEE->pDB->GetSOMForObject(pObject_block->get_object_type_name(),
				 id_d_O,
				 som))
      throw CEMdFDBDBError();

    try {
      // Update local symbol table with information
      pObject_block->get_object()->retrieve_feature_values(pEE->pDB, pObject_block->get_object_type_name(), id_d_O);
    } catch (CEMdFDBException e) {
      throw e;
    }

    // Only do rest of checking if rules are obeyed
    if (((((fl == kLast) 
	   || 
	   (fl == kFirstAndLast))
	  && 
	  (som.last() == Su.last()))
	 || 
	 (fl == kNoFirstLast))
	&&
	(som.part_of(Su))
	&&
	R_features(pEE, pObject_block->get_object(), 
		   pObject_block->get_feature_constraints())) {

      // So we passed the test

      try {
	// Update symbol table with object reference declaration
	R_object_reference_declaration_DB(pEE, id_d_O, pObject_block->get_object_reference_index());
      } catch (CEMdFDBException e) {
	throw e;
      }


      // Make new universe (O.first()..O.last())
      CSet_of_monad_ms Uping(som.first(),som.last());
      // Hat(som, Uping);
      
      // Calculate new inst
      CInst* pInstping;
      pInstping = R_inst_opt_blocks(pEE, Uping, pObject_block->get_opt_blocks());

      // Compute the inner blocks
      CSheaf* pInner;
      try {
	pInner = R_opt_blocks(pEE, Uping, som, pInstping, pObject_block->get_opt_blocks());
      } catch (CEMdFDBException e) {
	if (pInstping != 0 && !pInstping->is_aggregate())
	  delete pInstping;
	throw e;
      }
      
      // Clean up pInstping
      if (pInstping != 0 && !pInstping->is_aggregate())
	delete pInstping;

      // Only return the object if the inner was empty or did not fail
      if (pInner == 0 || !pInner->is_fail()) {
	CMatched_object* pO;
	eRetrieval retrieval = pObject_block->get_retrieval();
	bool bNoRetrieve = R_retrieval_retrieval(pEE, retrieval);
	bool bFocus = R_focus_retrieval(pEE, retrieval);
	pO = new CMatched_object(id_d_O, pObject_block->get_object_type_index(), som, bFocus, pInner);
	pLOMN->append(CMo_nr(pO, bNoRetrieve));
      } else {
	// Clean up pInner
	delete pInner;
      }
    }
    ++ci;
  }

  // If the lomn is empty at this point, insert ( NIL_mo, true )
  if (pLOMN->begin() == pLOMN->end()) {
    CMatched_object* pMo;
    pMo = new CMatched_object(kMOKNIL_mo);
    pLOMN->append(CMo_nr(pMo, true));
  }
}

void R_object_block_inst(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CObject_block* pObject_block, CList_of_mo_nr* pLOMN)
{
  // Get range
  CInst::const_iterator ci, ciend;
  pObject_block->m_inst->objects_at_Sm(Sm, ci, ciend);
  while (ci != ciend) {
    // Get object
    CInstObject *pInstObject;
    pInstObject = ci->second;

    // Get id_d of O
    id_d_t id_d_O = pInstObject->get_id_d();

    // Get last/first_and_last
    eFirstLast fl = pObject_block->get_first_last();

    // Get set of monads for object
    const CSet_of_monad_ms& som = pInstObject->get_monads();

    // Update local symbol table with information
    CMQLObject *pObj = pObject_block->get_object();
    pObj->retrieve_feature_values(pInstObject);

    // Only do rest of checking if rules are obeyed
    if (((((fl == kLast) 
	   || 
	   (fl == kFirstAndLast))
	  && 
	  (som.last() == Su.last()))
	 || 
	 (fl == kNoFirstLast))
	&&
	(som.part_of(Su))
	&&
	R_features(pEE, pObject_block->get_object(), 
		   pObject_block->get_feature_constraints())) {

      // So we passed the test

      try {
	// Update symbol table with object reference declaration
	R_object_reference_declaration_inst(pEE, id_d_O, pObject_block->get_object(), pObject_block->get_object_reference_index());
	// R_object_reference_declaration_DB(pEE, id_d_O, pObject_block->get_object_reference_index());
      } catch (CEMdFDBException e) {
	throw e;
      }


      // Make new universe (O.first()..O.last())
      CSet_of_monad_ms Uping(som.first(),som.last());
      //Hat(som, Uping);
      
      // Calculate new inst
      CInst* pInstping;
      pInstping = R_inst_opt_blocks(pEE, Uping, pObject_block->get_opt_blocks());

      // Compute the inner blocks
      CSheaf* pInner;
      try {
	pInner = R_opt_blocks(pEE, Uping, som, pInstping, pObject_block->get_opt_blocks());
      } catch (CEMdFDBException e) {
	if (pInstping != 0 && !pInstping->is_aggregate())
	  delete pInstping;
	throw e;
      }
      
      // Clean up pInstping
      if (pInstping != 0 && !pInstping->is_aggregate())
	delete pInstping;

      // Only return the object if the inner was empty or did not fail
      if (pInner == 0 || !pInner->is_fail()) {
	CMatched_object* pO;
	eRetrieval retrieval = pObject_block->get_retrieval();
	bool bNoRetrieve = R_retrieval_retrieval(pEE, retrieval);
	bool bFocus = R_focus_retrieval(pEE, retrieval);
	pO = new CMatched_object(id_d_O, pObject_block->get_object_type_index(), som, bFocus, pInner);
	pLOMN->append(CMo_nr(pO, bNoRetrieve));
      } else {
	// Clean up pInner
	delete pInner;
      }
    }
    ++ci;
  }

  // If the lomn is empty at this point, insert ( NIL_mo, true )
  if (pLOMN->begin() == pLOMN->end()) {
    CMatched_object* pMo;
    pMo = new CMatched_object(kMOKNIL_mo);
    pLOMN->append(CMo_nr(pMo, true));
  }
}

void R_object_block(CMQL_execution_environment *pEE, const CSet_of_monad_ms& U, const CSet_of_monad_ms& Su, monad_m Sm, CObject_block* pObject_block, CList_of_mo_nr* pLOMN)
{
  // Choose method
  if (pObject_block->m_inst == 0) {
    R_object_block_DB(pEE, U, Su, Sm, pObject_block, pLOMN);
  } else {
    R_object_block_inst(pEE, U, Su, Sm, pObject_block, pLOMN);
  }
}


