/*
 * mql_database_statements
 *
 * MQL DATABASE statements
 *
 * Ulrik Petersen
 * Created: 3/1-2001 (1st of March, 2001)
 * Last update: 11/22-2001
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/


#include <mql_database_statements.h>
#include <emdfdb.h>
#include <mql_error.h>
#include <mql_execution_environment.h>


//////////////////////////////////////////////////////////////
//
// CDatabaseStatement 
// base class for DATABASE statements
//
//////////////////////////////////////////////////////////////
CDatabaseStatement::~CDatabaseStatement()
{
  delete m_database_name;
}


////////////////////////////////////////////////////////////////
//
// CREATE DATABASE
//
// CCreateDatabaseStatement 
//
//////////////////////////////////////////////////////////////
bool CCreateDatabaseStatement::Exec()
{
  // Create database
  return m_pEE->pDB->CreateDatabase(*m_database_name);
}


////////////////////////////////////////////////////////////////
//
// USE DATABASE
//
// CUseStatement 
//
//////////////////////////////////////////////////////////////

bool CUseStatement::Exec()
{
  // Use database
  bool bResult = m_pEE->pDB->UseDatabase(*m_database_name);

  // Fill m_pEE->m_object_type_names with new object types.
  if (bResult) {
    m_pEE->fillOTNs();
  }

  // Return result
  return bResult;
}


////////////////////////////////////////////////////////////////
//
// DROP DATABASE
//
// CDropDatabaseStatement 
//
//////////////////////////////////////////////////////////////

bool CDropDatabaseStatement::Exec()
{
  // Drop database
  return m_pEE->pDB->DropDatabase(*m_database_name);
}


////////////////////////////////////////////////////////////////
//
// VACUUM DATABASE
//
// CVacuumDatabaseStatement 
//
//////////////////////////////////////////////////////////////

bool CVacuumDatabaseStatement::Exec()
{
  // Drop database
  return m_pEE->pDB->Vacuum(m_bAnalyze);
}


