/*
 * libmqlperl.i: SWIG interface to libmql.la from Perl
 *   by Chris Wilson <emdros@qwirx.com>, 
 * Created: Late December 2002
 * Last update: 3/18-2003
 *
 *   stolen from:
 * libemdfruby.i: SWIG interface to libemdf.la
 *   by Kirk Lowery <klowery@wts.edu>
 *   Copyright (C) 2002   Ulrik Petersen
 *   Mon Oct 28 10:09:52 EST 2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/

/* mql execution environment */

%{
#undef list
#include "../../include/mql_error.h"
#include "../../include/mql_execute.h"
#include "../../include/mql_types.h"
#include "../../include/mql_sheaf.h"
#include "../../include/mql_result.h"
#include "../../include/mql_execution_environment.h"
#include "../../include/emdros_environment.h"
%}


%inline %{
#include <iostream>
%}


%include "../../include/mql_execution_environment.h"
%include "../../include/mql_error.h"
%include "../../include/mql_execute.h"
%include "../../include/mql_types.h"
%include "../../include/mql_result.h"
%include "../../include/emdros_environment.h"


/*
typedef std::list MOList; 
*/


/* typemap to convert the CStraw* returned by CStraw_to_array
 * into a list of CMatchedObject objects */

%typemap(out) CStraw* {
	CStraw* straw = $1;

	/* count the number of MOs in this straw */
	int total_num_mos = 0;
	StrawConstIterator mo_iter;
	for (mo_iter = straw->const_iterator(); mo_iter.hasNext(); mo_iter.next()) {
		total_num_mos++;
	}

	SV** mo_svs = new SV* [total_num_mos];

	int this_mo_num = 0;
	for (mo_iter = straw->const_iterator(); mo_iter.hasNext(); mo_iter.next()) {
		SV* this_mo_sv = sv_newmortal();
		CMatched_object* mo_copy =
			new CMatched_object(*(mo_iter.current()));
		sv_setref_pv(this_mo_sv, SWIGTYPE_p_CMatched_object->name, 
			mo_copy);
		mo_svs[this_mo_num++] = this_mo_sv;
	}

	AV* all_mos_av = av_make(total_num_mos, mo_svs);
	delete mo_svs;

	$result = newRV((SV*)all_mos_av);
	sv_2mortal($result);
	argvi++;
}

// this function doesn't do anything except activate the above typemap

%inline %{
CStraw* CStraw_to_array (CStraw* straw) {
	return straw;
}
%}




/* typemap to convert the CList_of_straws returned by CSheaf::get_plist
 * into a list of CStraw objects */

%typemap(out) CList_of_straws* {
	CList_of_straws* straws = $1;

	/* count the number of straws in this sheaf */
	int total_num_straws = 0;
	SheafIterator straw_iter;

	if (straws != NULL) {
		for (straw_iter = straws->iterator(); straw_iter.hasNext();
			straw_iter.next()) {
			total_num_straws++;
		}
	}

	SV** straw_svs = new SV* [total_num_straws];

	if (straws != NULL) {
		int this_straw_num = 0;
		for (straw_iter = straws->iterator(); straw_iter.hasNext();
			straw_iter.next()) {
			SV* this_straw_sv = sv_newmortal();
			CStraw* straw_copy = new CStraw(*(straw_iter.current()));
			sv_setref_pv(this_straw_sv, SWIGTYPE_p_CStraw->name, 
				straw_copy);
			straw_svs[this_straw_num++] = this_straw_sv;
		}
	}

	AV* all_straws_av = av_make(total_num_straws, straw_svs);
	delete straw_svs;

	$result = newRV((SV*)all_straws_av);
	sv_2mortal($result);
	argvi++;
}

%include "../../include/mql_sheaf.h"

