/*
 * emdf_output.h
 *
 * Class to output something either in XML or in console-format.
 *
 * Ulrik Petersen
 * Created: 10/21-2001
 * Last update: 3/16-2004
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2004  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef EMDF_OUTPUT__H__
#define EMDF_OUTPUT__H__


#ifndef SWIG
#include <emdros-lconfig.h>

#if HAVE_OSTREAM
#include <ostream>
#else
#include <ostream.h>
#endif

#include <string>
#include <list>
#endif

class CEMdFOutputException {};
class CEMdFOutputWrongCharactersetException : public CEMdFOutputException {};

typedef enum {
  kCSASCII,
  kCSISO_8859_1,
  kCSISO_8859_8,
  kCSUTF8,
  kCSISO_8859_2,
  kCSISO_8859_3,
  kCSISO_8859_4,
  kCSISO_8859_5,
  kCSISO_8859_6,
  kCSISO_8859_7,
  kCSISO_8859_9,
  kCSISO_8859_10,
  kCSISO_8859_13,
  kCSISO_8859_14,
  kCSISO_8859_15,
  kCSISO_8859_16
} eCharsets;

typedef enum {
  kOKXML,
  kOKConsole
} eOutputKind;

#ifndef SWIG
typedef std::pair<std::string, std::string> AttributePair;
typedef std::list<AttributePair> AttributePairList;
#endif

class CEMdFOutput {
protected:
  eCharsets m_charset;
  eOutputKind m_output_kind;
  std::ostream *m_pStream;
  int m_current_indent;
  int m_indent_chars;
public:
  CEMdFOutput(eCharsets charset, std::ostream *pStream, eOutputKind output_kind, int indent_chars = 3);
  ~CEMdFOutput();
  // Getting
  eOutputKind get_output_kind(void) const { return m_output_kind; }
  void newline();
#ifndef SWIG
  // Output
  void increase_indent();
  void decrease_indent();
  void out(std::string s);
  void out_char_data(std::string s);
  void flush() { *m_pStream << std::flush; };
  // XML members
  void print_XMLDecl();
  void print_DTD_start(std::string root_element);
  void print_DTD_end();
  void start_tag(std::string name, bool newline_before = false);
  void start_tag(std::string name, const AttributePairList& attributes, bool newline_before = false); // Must have pairs of (attribute name, attribute value) 
  void start_single_tag(std::string name, bool newline_before = false); // for tags of type <tag/>
  void start_single_tag(std::string name, const AttributePairList& attributes, bool newline_before = false); // for tags of type <tag/>
  void end_tag(std::string name, bool newline_before = false);
protected:
  void emit_attributes(const AttributePairList& attributes);
#endif // !defined SWIG
};

#endif /* EMDF_OUTPUT__H__ */
