/*
 * emdf_value.h
 *
 * CEMdFValue class
 *
 * Ulrik Petersen
 * Created: 1/18-2003
 * Last update: 6/23-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/



#ifndef EMDF_VALUE__H__
#define EMDF_VALUE__H__

#include <emdf.h>
#include <string>

typedef enum { kEqual, kLessThan, kGreaterThan, kNotEqual, kLessThanOrEqual, kGreaterThanOrEqual, kTilde, kNotTilde } eComparisonOp;

extern std::string eComparisonOp2SQLString(eComparisonOp e);



typedef enum {
  kEVInt,
  kEVEnum,
  kEVID_D,
  kEVString
} eEVkind;

class CEMdFValue {
private:
  union {
    id_d_t m_id_d;
    long m_int;
    std::string *m_pString;
  } m_u;
  eEVkind m_kind;
public: 
  CEMdFValue(); // Don't use unless you plan to = assign a new value later
  CEMdFValue(const CEMdFValue& other);
  CEMdFValue(eEVkind kind, long i);
  CEMdFValue(const std::string& str);
  ~CEMdFValue();
  eEVkind get_kind(void) const { return m_kind; };
  const std::string get_string(void) const { return *m_u.m_pString; };
  id_d_t get_id_d(void) const { return m_u.m_id_d; };
  long get_int(void) const { return m_u.m_int; };
  long get_enum(void) const { return m_u.m_int; };
  bool compare(const CEMdFValue& other, eComparisonOp op) const;
  void get_as_string(std::string& result) const;
  CEMdFValue& operator=(const CEMdFValue& other);
private:
  bool compare_int(const CEMdFValue& other, eComparisonOp op) const;
  bool compare_string(const CEMdFValue& other, eComparisonOp op) const;
  void assign(const CEMdFValue& other);
  void delete_contents();
};




#endif // EMDF_VALUE__H__
