/*
 * mql_database_statements.h
 *
 * Definitions of AST-classes for MQL
 *
 * Ulrik Petersen
 * Created: 11/22-2002
 * Last update: 11/22-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_DATABASE_STATEMENTS__H__
#define MQL_DATABASE_STATEMENTS__H__

#include <mql_types.h>
#include <mql_execution_environment.h>

// Base class for CREATE, USE, and DROP DATABASE
class CDatabaseStatement : public CStatement {
protected:
  std::string* m_database_name;
public:
  CDatabaseStatement(CMQL_execution_environment *pEE, std::string* database_name) : CStatement(pEE), m_database_name(database_name)  {};
  virtual ~CDatabaseStatement();
};

// CREATE DATABASE
class CCreateDatabaseStatement : public CDatabaseStatement {
public:
  CCreateDatabaseStatement(CMQL_execution_environment *pEE, std::string* database_name) : CDatabaseStatement(pEE, database_name) {};
  virtual ~CCreateDatabaseStatement() {};
  virtual bool Exec();
};

// USE DATABASE
class CUseStatement : public CDatabaseStatement {
public:
  CUseStatement(CMQL_execution_environment *pEE, std::string* database_name) : CDatabaseStatement(pEE, database_name) {};
  virtual ~CUseStatement() {};
  virtual bool Exec();
};

// DROP DATABASE
class CDropDatabaseStatement : public CDatabaseStatement {
public:
  CDropDatabaseStatement(CMQL_execution_environment *pEE, std::string* database_name) : CDatabaseStatement(pEE, database_name) {};
  virtual ~CDropDatabaseStatement() {};
  virtual bool Exec();
};

// VACUUM DATABASE
class CVacuumDatabaseStatement : public CStatement {
private:
  bool m_bAnalyze;
public:
  CVacuumDatabaseStatement(CMQL_execution_environment *pEE, bool bAnalyze) : CStatement(pEE), m_bAnalyze(bAnalyze) {};
  virtual ~CVacuumDatabaseStatement() {};
  virtual bool Exec();
};



#endif /* MQL_DATABASE_STATEMENTS__H__ */
