/*
 * mql_enumeration_statements.h
 *
 * Definitions of AST-classes for MQL
 *
 * Ulrik Petersen
 * Created: 11/22-2002
 * Last update: 11/22-2002
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2002  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_ENUMERATION_STATEMENTS__H__
#define MQL_ENUMERATION_STATEMENTS__H__

#include <mql_types.h>
#include <mql_execution_environment.h>

class CEnumerationStatement : public CStatement {
protected:
  std::string* m_enumeration_name;
  id_d_t m_enum_id;
public:
  CEnumerationStatement(CMQL_execution_environment *pEE, 
			std::string* enumeration_name) : CStatement(pEE), 
    m_enumeration_name(enumeration_name) {};
  virtual ~CEnumerationStatement();
  bool SymbolEnumerationExists(bool& bResult); // Gets the enum_id
};

// Helper class ec_declaration
class CECDeclaration {
private:
  CECDeclaration* m_next;
  bool m_is_default;
  std::string* m_ec_name;
  long* m_opt_ec_initialization;
  long m_value;
public:
  CECDeclaration(bool is_default, std::string* ec_name, long* opt_ec_initialization, CECDeclaration* next);
  virtual ~CECDeclaration();
  void WeedDefault(CMQL_execution_environment *pEE, bool& bResult);
  bool get_is_default();
  bool has_opt_ec_initialization() { return m_opt_ec_initialization != 0; };
  long get_opt_ec_initialization();
  const std::string& get_ec_name();
  long get_value(void) { return m_value; };
  CECDeclaration* Next() { return m_next; };
  void set_Next(CECDeclaration* next) { m_next = next; };
  bool SymbolExistenceOfConstants(CMQL_execution_environment *pEE, id_d_t enum_id, bool& bResult);
  void SymbolAssignValues(void);
  bool CreateConstants(CMQL_execution_environment *pEE, id_d_t enum_id);
private:
  void WeedDefaultPrivate(CMQL_execution_environment *pEE, bool& bResult, bool& has_met_default);
  void SymbolAssignValuesPrivate(long& previous);
};

// CREATE ENUMERATION
class CCreateEnumerationStatement : public CEnumerationStatement {
protected:
  CECDeclaration* m_ec_declarations;
public:
  CCreateEnumerationStatement(CMQL_execution_environment *pEE, 
			      std::string* enumeration_name, 
			      CECDeclaration* ec_declaration_list);
  virtual ~CCreateEnumerationStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// Helper class: ec_update
class CECUpdate {
private:
  CECUpdate* m_next;
  eAddUpdateRemove m_kind;
  std::string* m_ec_name;
  long m_ec_initialization;
  bool m_is_default;
public:
  CECUpdate(bool is_default, 
	    std::string* ec_name, 
	    long ec_initialization, 
	    eAddUpdateRemove kind, 
	    CECUpdate* next);
  virtual ~CECUpdate();
  void WeedDefault(CMQL_execution_environment *pEE, bool& bResult);
  bool get_is_default() { return m_is_default; };
  long get_ec_initialization() { return m_ec_initialization; };
  const std::string& get_ec_name() { return *m_ec_name; };
  eAddUpdateRemove get_kind() { return m_kind; };
  CECUpdate* Next() { return m_next; };
  void set_Next(CECUpdate* next) { m_next = next; };
  bool SymbolConstantExistence(CMQL_execution_environment *pEE, id_d_t enum_id, bool& bResult);
  bool SymbolDefaultIsRemoved(CMQL_execution_environment *pEE, id_d_t enum_id, bool& bDefaultIsRemoved);
  bool has_default(void);
  const std::string& get_default(void) const; // Assumes that one is the default
  bool ExecAddUpdateRemove(CMQL_execution_environment *pEE, id_d_t enum_id);
  
private:
  void WeedDefaultPrivate(CMQL_execution_environment *pEE, bool& bResult, bool& has_met_default);
};

// UPDATE ENUMERATION
class CUpdateEnumerationStatement : public CEnumerationStatement {
private:
  CECUpdate* m_ec_update_list;
public:
  CUpdateEnumerationStatement(CMQL_execution_environment *pEE, 
			      std::string* enumeration_name,
			      CECUpdate* ec_update_list);
  virtual ~CUpdateEnumerationStatement();
  virtual void Weed(bool& bResult);
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};

// DROP ENUMERATION
class CDropEnumerationStatement : public CEnumerationStatement {
public:
  CDropEnumerationStatement(CMQL_execution_environment *pEE, 
			    std::string* enumeration_name);
  virtual ~CDropEnumerationStatement();
  virtual bool Symbol(bool& bResult);
  virtual bool Exec();
};





#endif /* MQL_ENUMERATION_STATEMENTS__H__ */
