/*
 * mql_query.h
 *
 * Definitions of AST-classes for MQL Query
 *
 * Ulrik Petersen
 * Created: 2/27-2001
 * Last update: 6/19-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_QUERY_TYPES__H__
#define MQL_QUERY_TYPES__H__

#include <mql_types.h>
#include <emdf.h>
#include <emdf_value.h>
#include <infos.h>
#include <string>
#include <mql_object.h>
#include <pcre.h>
#include <map>

class CObject_block_common; // Forward declaration
class CSet_of_monad_ms; // Forward declaration

typedef std::multimap<std::string, CObject_block_common*> String2COBPtrMMap;


class CMQL_execution_environment; // Forward declaration
class CBlocks; // Forward declaration

typedef enum { kNoRetrieve, kRetrieve, kRetrieveFocus } eRetrieval;

class CGap_block {
protected:
  eRetrieval m_retrieval;
  CBlocks* m_opt_blocks;
public:
  CGap_block(eRetrieval retrieval, CBlocks *opt_blocks);
  virtual ~CGap_block();
  eRetrieval get_retrieval() { return m_retrieval; };
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  CBlocks* get_opt_blocks() { return m_opt_blocks; };
};

class COpt_gap_block : public CGap_block {
public:
  COpt_gap_block(eRetrieval retrieval, CBlocks *opt_blocks) : CGap_block(retrieval, opt_blocks) {};
  virtual ~COpt_gap_block() {};
};

typedef enum { kOptGapBlock, kGapBlock, kObjectBlock } eBlockKind;

class CObject_block; // forward declaration

class CBlock {
private:
  eBlockKind m_kind;
  CGap_block* m_gap_block;
  COpt_gap_block* m_opt_gap_block;
  CObject_block* m_object_block;
public:
  CBlock(COpt_gap_block* opt_gap_block);
  CBlock(CGap_block* gap_block);
  CBlock(CObject_block* object_block);
  virtual ~CBlock();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void WeedFirstLast(CMQL_execution_environment *pEE, bool& bResult, bool is_first, bool is_last);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  eBlockKind get_kind() { return m_kind; };
  bool is_gap_or_opt_gap_block(void); // Is it an opt_gap_block or a gap_block?
  CGap_block* get_gap_block(void) { return m_gap_block; };
  COpt_gap_block* get_opt_gap_block(void) { return m_opt_gap_block; };
  CObject_block* get_object_block(void) { return m_object_block; };
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set, String2COBPtrMMap& mmap);
  bool HasFirstLast() const;
  void CalculateMMap(String2COBPtrMMap& mmap);
};

class CRest_of_block_str {
private:
  CBlock* m_block;
  CRest_of_block_str* m_rest_of_block_str;
public:
  CRest_of_block_str(CBlock* block);
  CRest_of_block_str(CBlock* block, CRest_of_block_str* rest_of_block_str);
  virtual ~CRest_of_block_str();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void WeedFirstLast(CMQL_execution_environment *pEE, bool& bResult, bool is_first, bool is_last);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_block() { return m_rest_of_block_str == 0; };
  CBlock* get_block() { return m_block; };
  CRest_of_block_str* get_rest_of_block_str() { return m_rest_of_block_str; };
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set, String2COBPtrMMap& mmap);
  bool HasFirstLast() const;
  void CalculateMMap(String2COBPtrMMap& mmap);
};

class CObject_reference_usage {
private:
  std::string* m_object_reference;
  std::string* m_feature_name;
  feature_info m_feature_info;
  short int m_index_in_symboltable;
  short int m_index_in_symboltable_entry;
public:
  CObject_reference_usage(std::string* object_reference, std::string* feature_name);
  virtual ~CObject_reference_usage();
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  const std::string& get_object_reference() { return *m_object_reference; };
  const std::string& get_feature_name() { return *m_feature_name; };
  short int get_index_in_symboltable(void) const { return m_index_in_symboltable; };
  short int get_index_in_symboltable_entry(void) const { return m_index_in_symboltable_entry; };
};

typedef enum { kValEnumConst, kValInteger, kValString, kValObjectReferenceUsage } eValueKind;

class CValue {
private:
  CEMdFValue *m_emdf_value;
  CObject_reference_usage* m_object_reference_usage;
  std::string* m_enum_const;
  long m_integer;
  std::string* m_string;
  eValueKind m_kind;
  id_d_t m_enum_id;
  long m_enum_const_value;
public:
  CValue(std::string* str, eValueKind kind); // for enum_const and string
  CValue(long integer);
  CValue(CObject_reference_usage* object_reference_usage);
  virtual ~CValue();
  bool Symbol(CMQL_execution_environment *pEE, id_d_t feature_type_id, bool& bResult);
  eValueKind get_kind() { return m_kind; };
  const std::string& get_enum_const();
  std::string get_as_string(CMQL_execution_environment *pEE) const;
  long get_integer();
  const std::string& get_string();
  const CEMdFValue* get_emdf_value(void);
  CObject_reference_usage* get_object_reference_usage();
  id_d_t get_enum_id(void) const { return m_enum_id; };
};

class CFeature_comparison {
private:
  std::string* m_feature_name;
  feature_info m_feature_info;
  eComparisonOp m_comparison_op;
  CValue* m_value;
  pcre *m_pcre;
  pcre_extra *m_pcre_extra;
  int *m_ovector;
  int m_ovectorsize;
  short int m_feature_index;
public:
  CFeature_comparison(std::string* feature_name,
		      eComparisonOp comparison_op,
		      CValue* value);
  virtual ~CFeature_comparison();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, id_d_t enclosing_object_type_id, bool& bResult);
  void SymbolAddFeatures(CMQLObject* pObj);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  const std::string& get_feature_name() { return *m_feature_name; };
  eComparisonOp get_comparison_op() { return m_comparison_op; };
  CValue* get_value() { return m_value; };
  bool compare(CMQL_execution_environment *pEE, const CEMdFValue *left_value);
  std::string make_constraints(bool bContextIsPositive) const;
  short int get_feature_index(void) const { return m_feature_index; };
};


class CFFeatures; // Forward declaration

class CFFactor {
private:
  CFFactor* m_ffactor;
  CFFeatures* m_ffeatures;
  CFeature_comparison* m_feature_comparison;
public:
  CFFactor(CFFactor* ffactor);
  CFFactor(CFFeatures* ffeatures);
  CFFactor(CFeature_comparison* feature_comparison);
  virtual ~CFFactor();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, id_d_t enclosing_object_type_id, bool& bResult);
  void SymbolAddFeatures(CMQLObject* pObj);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_NOT() const { return m_ffactor != 0; };
  bool is_parenthesis() const { return m_ffeatures != 0; };
  bool is_feature_comparison() const { return m_feature_comparison != 0; };
  CFFactor* get_ffactor() { return m_ffactor; };
  CFFeatures* get_ffeatures() { return m_ffeatures; };
  CFeature_comparison* get_feature_comparison() { return m_feature_comparison; };
  std::string make_constraints(bool bContextIsPositive) const;
};

class CFTerm {
private:
  CFFactor* m_ffactor;
  CFTerm* m_fterm;
public:
  CFTerm(CFFactor* ffactor);
  CFTerm(CFFactor* ffactor, CFTerm* fterm);
  virtual ~CFTerm();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, id_d_t enclosing_object_type_id, bool& bResult);
  void SymbolAddFeatures(CMQLObject* pObj);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_ffactor() const { return m_fterm == 0; };
  CFFactor* get_ffactor() { return m_ffactor; };
  CFTerm* get_fterm() { return m_fterm; };
  std::string make_constraints(bool bContextIsPositive) const;
};

class CFFeatures {
private:
  CFTerm* m_fterm;
  CFFeatures* m_ffeatures;
  std::string *m_constraints;
public:
  CFFeatures(CFTerm* fterm);
  CFFeatures(CFFeatures* ffeatures, CFTerm* fterm);
  virtual ~CFFeatures();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, id_d_t enclosing_object_type_id, bool& bResult);
  void SymbolAddFeatures(CMQLObject* pObj);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_fterm() const { return m_ffeatures == 0; };
  CFTerm* get_fterm() { return m_fterm; };
  CFFeatures* get_ffeatures() { return m_ffeatures; };
  const std::string& get_constraints(void);
  std::string make_constraints(bool bContextIsPositive) const;
};

typedef enum { kFirst, kLast, kFirstAndLast, kNoFirstLast } eFirstLast;
class CBlocks; // Forward declaration

class CObject_block_common {
protected:
  std::string* m_object_type_name;
  std::string* m_object_reference;
  eRetrieval m_retrieval;
  eFirstLast m_first_last;
  CFFeatures* m_feature_constraints;
  CFeature *m_pFeature_retrieval;
  CBlocks* m_opt_blocks;
  id_d_t m_object_type_id;
  short int m_object_type_index;
  std::string m_pre_query_string;
  std::string m_characteristic_string;
  CMQLObject* m_object;
  short int m_object_reference_index;
public:
  CInst *m_inst;
  CObject_block_common(std::string* object_type_name,
		       std::string* object_reference,
		       eRetrieval retrieval,
		       eFirstLast first_last,
		       CFFeatures* feature_constraints,
		       CFeature* feature_retrieval,
		       CBlocks* opt_blocks);
  virtual ~CObject_block_common();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void WeedFirstLast(CMQL_execution_environment *pEE, bool& bResult, bool is_first, bool is_last);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  const std::string& get_object_type_name() const { return *m_object_type_name; };
  id_d_t get_object_type_id() const { return m_object_type_id; };
  short int get_object_type_index() const { return m_object_type_index; };
  const std::string& get_object_reference() { assert(m_object_reference); return *m_object_reference; };
  eRetrieval get_retrieval() { return m_retrieval; };
  eFirstLast get_first_last() { return m_first_last; };
  const std::string& get_pre_query_string(void) const { return m_pre_query_string; };
  const std::string& get_characteristic_string(void) const { return m_characteristic_string; }
  short int get_object_reference_index(void) const { return m_object_reference_index; };
  void calculate_pre_query_string(void);
  void calculate_characteristic_string(void);
  CFFeatures* get_feature_constraints() { return m_feature_constraints; };
  CBlocks* get_opt_blocks() { return m_opt_blocks; };
  CMQLObject* get_object() { return m_object; };
  bool make_inst(CMQL_execution_environment *pEE, const CSet_of_monad_ms& Su, String2COBPtrMMap& mmap);
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set, String2COBPtrMMap& mmap);
  bool HasFirstLast() const;
  void CalculateMMap(String2COBPtrMMap& mmap);
  void set_object_reference_index(short int index) { m_object_reference_index = index; };
private:
  bool SymbolObjectTypeExists(CMQL_execution_environment *pEE, bool& bObjectTypeExists);
};

class CObject_block_first : public CObject_block_common {
public:
  CObject_block_first(std::string* object_type_name,
		      std::string* object_reference,
		      eRetrieval retrieval,
		      eFirstLast first_last,
		      CFFeatures* feature_constraints,
		      CFeature* feature_retrieval,
		      CBlocks* opt_blocks);
  virtual ~CObject_block_first();
};

class CObject_block : public CObject_block_common {
public:
  CObject_block(std::string* object_type_name,
		std::string* object_reference,
		eRetrieval retrieval,
		eFirstLast last,
		CFFeatures* feature_constraints,
		CFeature *pFeature_retrieval,
		CBlocks* opt_blocks);
  virtual ~CObject_block();
};


class CBlock_str {
private:
  CObject_block_first* m_object_block_first;
  CRest_of_block_str* m_rest_of_block_str;
public:
  CBlock_str(CObject_block_first* object_block_first);
  CBlock_str(CObject_block_first* object_block_first,
	     CRest_of_block_str* rest_of_block_str);
  virtual ~CBlock_str();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void WeedFirstLast(CMQL_execution_environment *pEE, bool& bResult, bool is_first, bool is_last);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_object_block_first() { return m_rest_of_block_str == 0; };
  CObject_block_first* get_object_block_first() { return m_object_block_first; };
  CRest_of_block_str* get_rest_of_block_str() { return m_rest_of_block_str; };
  const std::string& get_leftmost_object_type_name();
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set, String2COBPtrMMap& mmap);
  bool HasFirstLast() const;
  void CalculateMMap(String2COBPtrMMap& mmap);
};

class CPower {
private:
  long m_limit;
public:
  CPower(); // m_limit is 0
  CPower(long limit);
  virtual ~CPower();
  long get_limit() { return m_limit; };
};

class CBlock_string {
private:
  CBlock_str* m_block_str;
  CPower* m_power;
  CBlock_string* m_block_string;
public:
  CBlock_string(CBlock_str* block_str);
  CBlock_string(CBlock_str* block_str, CPower* power, CBlock_string* block_string);
  virtual ~CBlock_string();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void WeedFirstLast(CMQL_execution_environment *pEE, bool& bResult, bool is_first, bool is_last);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  bool is_block_str() { return m_power == 0; };
  CBlock_str* get_block_str() { return m_block_str; };
  CPower* get_power() { return m_power; };
  CBlock_string* get_block_string() { return m_block_string; };
  const std::string& get_leftmost_object_type_name();
  CObject_block_first *get_leftmost_object_block_first();
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set, String2COBPtrMMap& mmap);
  bool HasFirstLast() const;
  void CalculateMMap(String2COBPtrMMap& mmap);
};

class CBlocks {
private:
  CBlock_string* m_block_string;
  String2COBPtrMMap m_mmap;
public:
  CBlocks(CBlock_string* block_string);
  virtual ~CBlocks();
  void WeedTildeOperator(CMQL_execution_environment *pEE, bool& bResult);
  void Weed(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  CBlock_string* get_block_string() { return m_block_string; };
  const std::string& get_leftmost_object_type_name();
  CObject_block_first *get_leftmost_object_block_first();
  void CalculateMMap(void);
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set);
  bool HasFirstLast() const;
};

class CTopograph {
private:
  CBlocks* m_blocks;
public:
  CTopograph(CBlocks* blocks);
  virtual ~CTopograph();
  void Weed(CMQL_execution_environment *pEE, bool& bResult);
  bool Symbol(CMQL_execution_environment *pEE, bool& bResult);
  bool Type(CMQL_execution_environment *pEE, bool& bResult);
  CBlocks* get_blocks() { return m_blocks; };
  bool AggregateQuery(CMQL_execution_environment *pEE, const CSet_of_monad_ms& characteristic_set);
  bool HasFirstLast() const;
  void CalculateMMap(void);
};



#endif /* MQL_QUERY_TYPES__H__ */
