/*
 * mql_sheaf.h
 *
 * MQL sheafs
 *
 * Ulrik Petersen
 * Created: 3/8-2001
 * Last update: 6/21-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_SHEAF__H__
#define MQL_SHEAF__H__

#ifndef SWIG
#include <emdf.h>
#endif // !defined SWIG
#include <monads.h>
#include <emdf_output.h>
#include <string>
#include <list>


class CSheaf; // forward declaration
class CMQL_execution_environment; // forward declaration

typedef enum { kMOKNIL_mo, kMOKEMPTY_mo, kMOKID_D, kMOKID_M } eMOKind;

class CMatched_object {
private:
  CSet_of_monad_ms m_monads;
  id_d_t m_id_d;
  CSheaf* m_sheaf;
  eMOKind m_kind;
  bool m_focus;
  short int m_object_type_index;
  monad_m m_last;
  friend class CStraw;
public:  
#ifndef SWIG
  CMatched_object(eMOKind kind); // for kMOKNIL_mo, kMOKEMPTY_mo
  CMatched_object(id_d_t id_d, short int object_type_index, const CSet_of_monad_ms& monads, bool focus, CSheaf* sheaf); // for kMOKID_D
  CMatched_object(const CSet_of_monad_ms& monads, bool focus, CSheaf *sheaf); // for kMOKID_M
#endif // !defined SWIG
  CMatched_object(const CMatched_object& other);
  ~CMatched_object();
  const CSet_of_monad_ms& get_monads(void) const;
  id_d_t get_id_d(void) const;
  const CSheaf* get_sheaf(void) const;
  bool sheaf_is_empty(void) const { return m_sheaf == 0; };
  eMOKind get_kind(void) const { return m_kind; };
  bool get_focus(void) const { return m_focus; };
#ifndef SWIG
  CMatched_object& operator=(const CMatched_object& other);
#endif
  short int get_object_type_index(void) const { return m_object_type_index; };
  std::string getObjectTypeName(CMQL_execution_environment *pEE) const;
  void put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const;
  void put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const;
#ifndef SWIG
  monad_m get_last(void) const { return m_last; };
#endif // !defined SWIG
private:
  void copy_other(const CMatched_object& other);
};

#ifndef SWIG
typedef std::list<CMatched_object*> MOList;
#endif

class CStraw; // Forward declaration

class StrawConstIterator {
private:
  friend class CStraw;
  const CStraw *m_pMotherStraw;
  MOList::const_iterator m_iter;
public:
  StrawConstIterator(const CStraw *pMotherStraw);
  StrawConstIterator();
  StrawConstIterator(const StrawConstIterator& other);
  ~StrawConstIterator();
#ifndef SWIG
  StrawConstIterator& operator=(const StrawConstIterator& other);
#endif 
  bool hasNext() const; // Is the iterator == end iterator?  Doesn't alter iterator
  const CMatched_object* next(); // Gets current and advances iterator afterwards
  const CMatched_object* previous(); // Regresses iterator and then gets current
  const CMatched_object* current(); // Gets current without altering iterator
private:
  void assign(const StrawConstIterator& other);
};

class CStraw {
private:
  friend class StrawConstIterator;
  monad_m m_last;
  MOList m_list;
public:
#ifndef SWIG
  CStraw();
#endif // !defined SWIG
  ~CStraw();
  CStraw(const CStraw& other);
  StrawConstIterator const_iterator() const;
  monad_m get_last(void) const { return m_last; };
#ifndef SWIG
  CStraw& operator=(const CStraw& other);
  void append(CMatched_object* mo);
  void prepend(const CStraw& other);
  void update_last(const CMatched_object* mo);
#endif // !defined SWIG
  void put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const;
  void put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const;
private:
  void copy_other(const CStraw& other);
  void delete_mos(void);
};

#ifndef SWIG
typedef std::list<CStraw*> StrawList;
#endif

class CList_of_straws; // Forward declaration

class SheafIterator {
private:
  friend class CList_of_straws;
  CList_of_straws *m_pMotherList;
  StrawList::iterator m_iter;
public:
  SheafIterator(CList_of_straws *pMotherList);
  SheafIterator();
  SheafIterator(const SheafIterator& other);
  ~SheafIterator();
#ifndef SWIG
  SheafIterator& operator=(const SheafIterator& other);
#endif 
  bool hasNext() const; // Is the iterator == end iterator?  Doesn't alter iterator
  CStraw* next(); // Gets current and advances iterator afterwards
  CStraw* previous(); // Regresses iterator and then gets current
  CStraw* current(); // Gets current without altering iterator
private:
  void assign(const SheafIterator& other);
};



class SheafConstIterator {
private:
  friend class CList_of_straws;
  const CList_of_straws *m_pMotherList;
  StrawList::const_iterator m_iter;
public:
  SheafConstIterator(const CList_of_straws *pMotherList);
  SheafConstIterator();
  SheafConstIterator(const SheafConstIterator& other);
  ~SheafConstIterator();
#ifndef SWIG
  SheafConstIterator& operator=(const SheafConstIterator& other);
#endif 
  bool hasNext() const; // Is the iterator == end iterator?  Doesn't alter iterator
  const CStraw* next(); // Gets current and advances iterator afterwards
  const CStraw* previous(); // Regresses iterator and then gets current
  const CStraw* current(); // Gets current without altering iterator
private:
  void assign(const SheafConstIterator& other);
};



class CList_of_straws {
private:
  friend class SheafIterator;
  friend class SheafConstIterator;
  StrawList m_list;
public:
  bool m_success;
#ifndef SWIG
  CList_of_straws();
#endif // !defined SWIG
  ~CList_of_straws();
  CList_of_straws(const CList_of_straws& other);
  SheafIterator iterator();
  SheafConstIterator const_iterator() const;
#ifndef SWIG
  CList_of_straws& operator=(const CList_of_straws& other);
#endif // !defined SWIG
  bool is_empty() const { return m_list.begin() == m_list.end(); };
  void put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const;
  void put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const;
#ifndef SWIG
  void append(CStraw* straw);
  void append_and_subsume(CList_of_straws *pOther);
  void set_success(bool success) { m_success = success; };
  void Join(CStraw* s);
private:
  void copy_other(const CList_of_straws& other);
  void delete_straws(void);
#endif // !defined SWIG
};




class CSheaf {
private:
  CList_of_straws* m_plist;
public:
#ifndef SWIG
  // is_fail True:  Failed sheaf (no m_plist). 
  // is_fail False: Empty sheaf  (empty m_plist)
  CSheaf(bool is_fail); 
  CSheaf(CList_of_straws* plist);
#endif // !defined SWIG
  CSheaf(const CSheaf& other);
  ~CSheaf();
  bool is_fail(void) const;
  SheafIterator iterator();
  SheafConstIterator const_iterator() const;
  const CList_of_straws* get_plist(void) const { return m_plist; };
#ifndef SWIG
  CSheaf& operator=(const CSheaf& other);
#endif
  void put_Console(CEMdFOutput *pOut, CMQL_execution_environment *pEE) const;
  void put_XML(CEMdFOutput* pOut, CMQL_execution_environment *pEE) const;
  static void print_DTD(CEMdFOutput* pOut);
private:
  void copy_other(const CSheaf& other);
  void delete_straws(void);
};




#endif /* MQL_SHEAF__H__ */
