/*
 * mql_symboltable.h
 *
 * MQL symbol table for object references
 *
 * Ulrik Petersen
 * Created: 3/16-2001
 * Last update: 3/15-2003
 *
 */
/************************************************************************
 *
 *   Emdros - the database engine for analyzed or annotated text
 *   Copyright (C) 2001-2003  Ulrik Petersen
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, license version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Other licensing forms
 *   =====================
 *
 *   If you wish to negotiate commercial licensing, please contact
 *   Ulrik Petersen at ulrikp[at]users.sourceforge.net.
 *
 *   Licensing can also be negotiated if your organization is an
 *   educational, non-profit, charity, missionary or similar
 *   organization.
 *
 **************************************************************************/




#ifndef MQL_SYMBOLTABLE__H__
#define MQL_SYMBOLTABLE__H__

#include <emdf.h>
#include <emdfdb.h>
#include <inst_object.h>
#include <infos.h>
#include <string>
#include <string_func.h>
#include <map>
#include <list>
#include <emdf_value.h>
#include <smart_vector.h>
#include <functional>

class CMQLSymbolTableException {};
class CMQLSymbolTableEntryException {};

class CObject_block_common; // Forward declaration
class CMQLObject; // Forward declaration

typedef std::pair<CEMdFValue*, feature_info> ValPair;
struct valpair_equal : public std::equal_to<ValPair*> {
  bool operator() (const ValPair* x, const ValPair* y) const { return strcmp_nocase(x->second.get_name(),y->second.get_name()) == 0; };
};

typedef CSmartVector<ValPair, valpair_equal> ValPairVec;

class CMQLSymbolTableEntry {
private:
  id_d_t m_object_type_id;
  std::string m_object_type_name;
  ValPairVec m_vec;
  std::string m_object_reference;
  id_d_t m_id_d;
  std::list<feature_info> *m_pFeature_infos;
public:
  CMQLSymbolTableEntry(const std::string& object_reference,
		       const std::string& object_type_name,
		       id_d_t object_type_id);
  ~CMQLSymbolTableEntry();
  short int add_feature(const feature_info& fi);
  void set_feature(short int index, CEMdFValue *feature_value);
  const std::list<feature_info>& get_feature_infos(void);
  id_d_t get_feature_type_id(short int index);
  void get_feature_value(short int index, const CEMdFValue **result);
  void set_id_d(id_d_t id_d_O);
  id_d_t get_id_d(void) const { return m_id_d; };
  const std::string& get_object_reference(void) const { return m_object_reference; };
  const std::string& get_object_type_name(void) const { return m_object_type_name; };
  id_d_t get_object_type_id(void) const { return m_object_type_id; };
  void set_features(EMdFDB* pDB);
  void set_features(CMQLObject *pMQLObject);
};

// StrSTEPairVec
typedef std::pair<std::string, CMQLSymbolTableEntry*> StrSTEPair;
struct strste_pair_equal : public std::equal_to<StrSTEPair*> {
  bool operator() (const StrSTEPair* x, const StrSTEPair* y) const { return strcmp_nocase(x->first,y->first) == 0; };
};
typedef CSmartVector<StrSTEPair, strste_pair_equal> StrSTEPairVec;

// StrCOBCPairVec
typedef std::pair<std::string, CObject_block_common*> StrCOBCPair;
struct strcobc_pair_equal : public std::equal_to<StrCOBCPair*> {
  bool operator() (const StrCOBCPair* x, const StrCOBCPair* y) const { return strcmp_nocase(x->first,y->first) == 0; };
};
typedef CSmartVector<StrCOBCPair, strcobc_pair_equal> StrCOBCPairVec;

class CMQLSymbolTable {
private:
  StrSTEPairVec m_vec_ste;
  StrCOBCPairVec m_vec_object_block;
public:
  CMQLSymbolTable();
  ~CMQLSymbolTable();
  short int insert(CMQLSymbolTableEntry* pSTE, CObject_block_common *pObjectBlock);
  bool exists(const std::string& object_reference, short int& index);
  CMQLSymbolTableEntry* get(short int index);
  CObject_block_common* get_object_block(short int index);
  short int get_index(const std::string& object_reference);
};


#endif /* MQL_SYMBOLTABLE__H__ */
